@@ -12,14 +12,58 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+import importlib
 import logging
-from typing import TYPE_CHECKING, Callable, Dict, List, Tuple, Type
+import pkgutil
+import warnings
+from typing import TYPE_CHECKING, Callable, Dict, List, Sequence, Tuple, Type
+
+from toil.lib.compatibility import deprecated
+from toil.lib.memoize import memoize
 
 if TYPE_CHECKING:
     from toil.batchSystems.abstractBatchSystem import AbstractBatchSystem
 
 logger = logging.getLogger(__name__)
 
+#####
+# Plugin system/API
+#####
+
+def add_batch_system_factory(key: str, class_factory: Callable[[], Type['AbstractBatchSystem']]):
+    """
+    Adds a batch system to the registry for workflow or plugin-supplied batch systems.
+
+    :param class_factory: A function that returns a batch system class (NOT an instance), which implements :class:`toil.batchSystems.abstractBatchSystem.AbstractBatchSystem`.
+    """
+    _registry_keys.append(key)
+    _registry[key] = class_factory
+
+def get_batch_systems() -> Sequence[str]:
+    """
+    Get the names of all the availsble batch systems.
+    """
+    _load_all_plugins()
+
+    return _registry_keys
+
+def get_batch_system(key: str) -> Type['AbstractBatchSystem']:
+    """
+    Get a batch system class by name.
+
+    :raises: KeyError if the key is not the name of a batch system, and
+             ImportError if the batch system's class cannot be loaded.
+    """
+
+    return _registry[key]()
+
+
+DEFAULT_BATCH_SYSTEM = 'single_machine'
+
+#####
+# Built-in batch systems
+#####
+
 def aws_batch_batch_system_factory():
     from toil.batchSystems.awsBatch import AWSBatchBatchSystem
     return AWSBatchBatchSystem
@@ -29,11 +73,6 @@ def gridengine_batch_system_factory():
     return GridEngineBatchSystem
 
 
-def parasol_batch_system_factory():
-    from toil.batchSystems.parasol import ParasolBatchSystem
-    return ParasolBatchSystem
-
-
 def lsf_batch_system_factory():
     from toil.batchSystems.lsf import LSFBatchSystem
     return LSFBatchSystem
@@ -53,10 +92,6 @@ def slurm_batch_system_factory():
     from toil.batchSystems.slurm import SlurmBatchSystem
     return SlurmBatchSystem
 
-def tes_batch_system_factory():
-    from toil.batchSystems.tes import TESBatchSystem
-    return TESBatchSystem
-
 def torque_batch_system_factory():
     from toil.batchSystems.torque import TorqueBatchSystem
     return TorqueBatchSystem
@@ -71,29 +106,75 @@ def kubernetes_batch_system_factory():
     from toil.batchSystems.kubernetes import KubernetesBatchSystem
     return KubernetesBatchSystem
 
+#####
+# Registry implementation
+#####
 
-BATCH_SYSTEM_FACTORY_REGISTRY: Dict[str, Callable[[], Type["AbstractBatchSystem"]]] = {
+_registry: Dict[str, Callable[[], Type["AbstractBatchSystem"]]] = {
     'aws_batch'      : aws_batch_batch_system_factory,
-    'parasol'        : parasol_batch_system_factory,
     'single_machine' : single_machine_batch_system_factory,
     'grid_engine'    : gridengine_batch_system_factory,
     'lsf'            : lsf_batch_system_factory,
     'mesos'          : mesos_batch_system_factory,
     'slurm'          : slurm_batch_system_factory,
-    'tes'            : tes_batch_system_factory,
     'torque'         : torque_batch_system_factory,
     'htcondor'       : htcondor_batch_system_factory,
     'kubernetes'     : kubernetes_batch_system_factory
 }
-BATCH_SYSTEMS = list(BATCH_SYSTEM_FACTORY_REGISTRY.keys())
-DEFAULT_BATCH_SYSTEM = 'single_machine'
+_registry_keys = list(_registry.keys())
 
+# We will load any packages starting with this prefix and let them call
+# add_batch_system_factory()
+_PLUGIN_NAME_PREFIX = "toil_batch_system_"
+
+@memoize
+def _load_all_plugins() -> None:
+    """
+    Load all the batch system plugins that are installed.
+    """
+
+    for finder, name, is_pkg in pkgutil.iter_modules():
+        # For all installed packages
+        if name.startswith(_PLUGIN_NAME_PREFIX):
+            # If it is a Toil batch system plugin, import it
+            importlib.import_module(name)
+
+#####
+# Deprecated API
+#####
+
+# We used to directly access these constants, but now the Right Way to use this
+# module is add_batch_system_factory() to register and get_batch_systems() to
+# get the list/get_batch_system() to get a class by name.
+
+
+def __getattr__(name):
+    """
+    Implement a fallback attribute getter to handle deprecated constants.
+
+    See <https://stackoverflow.com/a/48242860>.
+    """
+    if name == "BATCH_SYSTEM_FACTORY_REGISTRY":
+        warnings.warn("BATCH_SYSTEM_FACTORY_REGISTRY is deprecated; use get_batch_system() or add_batch_system_factory()", DeprecationWarning)
+        return _registry
+    elif name == "BATCH_SYSTEMS":
+        warnings.warn("BATCH_SYSTEMS is deprecated; use get_batch_systems()", DeprecationWarning)
+        return _registry_keys
+    else:
+        raise AttributeError(f"Module {__name__} ahs no attribute {name}")
+
+
+@deprecated(new_function_name="add_batch_system_factory")
 def addBatchSystemFactory(key: str, batchSystemFactory: Callable[[], Type['AbstractBatchSystem']]):
     """
-    Adds a batch system to the registry for workflow-supplied batch systems.
+    Deprecated method to add a batch system.
     """
-    BATCH_SYSTEMS.append(key)
-    BATCH_SYSTEM_FACTORY_REGISTRY[key] = batchSystemFactory
+    return add_batch_system_factory(key, batchSystemFactory)
+
+
+#####
+# Testing utilities
+#####
 
 # We need a snapshot save/restore system for testing. We can't just tamper with
 # the globals because module-level globals are their own references, so we
@@ -106,7 +187,7 @@ def save_batch_system_plugin_state() -> Tuple[List[str], Dict[str, Callable[[],
     tests.
     """
 
-    snapshot = (list(BATCH_SYSTEMS), dict(BATCH_SYSTEM_FACTORY_REGISTRY))
+    snapshot = (list(_registry_keys), dict(_registry))
     return snapshot
 
 def restore_batch_system_plugin_state(snapshot: Tuple[List[str], Dict[str, Callable[[], Type['AbstractBatchSystem']]]]):
@@ -118,7 +199,7 @@ def restore_batch_system_plugin_state(snapshot: Tuple[List[str], Dict[str, Calla
     # We need to apply the snapshot without rebinding the names, because that
     # won't affect modules that imported the names.
     wanted_batch_systems, wanted_registry = snapshot
-    BATCH_SYSTEMS.clear()
-    BATCH_SYSTEMS.extend(wanted_batch_systems)
-    BATCH_SYSTEM_FACTORY_REGISTRY.clear()
-    BATCH_SYSTEM_FACTORY_REGISTRY.update(wanted_registry)
+    _registry_keys.clear()
+    _registry_keys.extend(wanted_batch_systems)
+    _registry.clear()
+    _registry.update(wanted_registry)
