@@ -28,22 +28,24 @@ import enlighten
 from toil import resolveEntryPoint
 from toil.batchSystems import DeadlockException
 from toil.batchSystems.abstractBatchSystem import (AbstractBatchSystem,
-                                                   BatchJobExitReason)
-from toil.bus import (JobAnnotationMessage,
-                      JobCompletedMessage,
+                                                   BatchJobExitReason,
+                                                   EXIT_STATUS_UNAVAILABLE_VALUE)
+from toil.bus import (JobCompletedMessage,
                       JobFailedMessage,
                       JobIssuedMessage,
                       JobMissingMessage,
                       JobUpdatedMessage,
-                      QueueSizeMessage)
-from toil.common import Config, Toil, ToilMetrics
+                      QueueSizeMessage,
+                      gen_message_bus_path,
+                      get_job_kind)
+from toil.common import Config, ToilMetrics
 from toil.cwl.utils import CWL_UNSUPPORTED_REQUIREMENT_EXIT_CODE
+from toil.exceptions import FailedJobsException
 from toil.job import (CheckpointJobDescription,
                       JobDescription,
                       ServiceJobDescription,
                       TemporaryID)
 from toil.jobStores.abstractJobStore import (AbstractJobStore,
-                                             NoSuchFileException,
                                              NoSuchJobException)
 from toil.lib.throttle import LocalThrottle
 from toil.provisioners.abstractProvisioner import AbstractProvisioner
@@ -51,7 +53,6 @@ from toil.provisioners.clusterScaler import ScalerThread
 from toil.serviceManager import ServiceManager
 from toil.statsAndLogging import StatsAndLogging
 from toil.toilState import ToilState
-from toil.exceptions import FailedJobsException
 
 logger = logging.getLogger(__name__)
 
@@ -115,10 +116,14 @@ class Leader:
         # state change information about jobs.
         self.toilState = ToilState(self.jobStore)
 
-        if self.config.write_messages is not None:
-            # Message bus messages need to go to the given file.
-            # Keep a reference to the return value so the listener stays alive.
-            self._message_subscription = self.toilState.bus.connect_output_file(self.config.write_messages)
+        if self.config.write_messages is None:
+            # The user hasn't specified a place for the message bus so we
+            # should make one.
+            self.config.write_messages = gen_message_bus_path()
+
+        # Message bus messages need to go to the given file.
+        # Keep a reference to the return value so the listener stays alive.
+        self._message_subscription = self.toilState.bus.connect_output_file(self.config.write_messages)
 
         # Connect to the message bus, so we will get all the messages of these
         # types in an inbox.
@@ -138,7 +143,8 @@ class Leader:
 
         # Batch system
         self.batchSystem = batchSystem
-        assert len(self.batchSystem.getIssuedBatchJobIDs()) == 0  # Batch system must start with no active jobs!
+        if len(self.batchSystem.getIssuedBatchJobIDs()) != 0:
+            raise RuntimeError("The initialized batchsystem did not start with 0 active jobs.")
         logger.debug("Checked batch system has no running jobs and no updated jobs")
 
         # Map of batch system IDs to job store IDs
@@ -370,7 +376,8 @@ class Leader:
 
         # If the successor job's predecessors have all not all completed then
         # ignore the successor as is not yet ready to run
-        assert len(successor.predecessorsFinished) <= successor.predecessorNumber
+        if len(successor.predecessorsFinished) > successor.predecessorNumber:
+            raise RuntimeError("There are more finished predecessors than possible.")
         if len(successor.predecessorsFinished) == successor.predecessorNumber:
             # All the successor's predecessors are done now.
             # Remove the successor job from the set of waiting multi-predecessor jobs.
@@ -391,8 +398,10 @@ class Leader:
         #Build map from successor to predecessors.
         if successor_id not in self.toilState.successor_to_predecessors:
             self.toilState.successor_to_predecessors[successor_id] = set()
-        assert isinstance(successor_id, str)
-        assert isinstance(predecessor_id, str)
+        if not isinstance(successor_id, str):
+            raise RuntimeError("The given successor ID is invalid.")
+        if not isinstance(predecessor_id, str):
+            raise RuntimeError("The given predecessor ID is invalid.")
         self.toilState.successor_to_predecessors[successor_id].add(predecessor_id)
 
         # Grab the successor
@@ -423,7 +432,8 @@ class Leader:
                      predecessor_id, len(next_successors))
         #Record the number of successors that must be completed before
         #the job can be considered again
-        assert self.toilState.count_pending_successors(predecessor_id) == 0, 'Attempted to schedule successors of the same job twice!'
+        if self.toilState.count_pending_successors(predecessor_id) != 0:
+            raise RuntimeError('Attempted to schedule successors of the same job twice!')
         self.toilState.successors_pending(predecessor_id, len(next_successors))
 
         # For each successor schedule if all predecessors have been completed
@@ -534,11 +544,13 @@ class Leader:
             # the job has services to run, which have not been started, start them
             # Build a map from the service jobs to the job and a map
             # of the services created for the job
-            assert readyJob.jobStoreID not in self.toilState.servicesIssued
+            if readyJob.jobStoreID in self.toilState.servicesIssued:
+                raise RuntimeError(f"The ready job: {readyJob.jobStoreID} was already issued.")
             self.toilState.servicesIssued[readyJob.jobStoreID] = set()
             for serviceJobList in readyJob.serviceHostIDsInBatches():
                 for serviceID in serviceJobList:
-                    assert serviceID not in self.toilState.service_to_client
+                    if serviceID in self.toilState.service_to_client:
+                        raise RuntimeError(f"The ready service ID: {serviceID} was already added.")
                     self.toilState.reset_job(serviceID)
                     serviceHost = self.toilState.get_job(serviceID)
                     self.toilState.service_to_client[serviceID] = readyJob.jobStoreID
@@ -675,7 +687,8 @@ class Leader:
             client = self.toilState.get_job(client_id)
 
             # Make sure services still want to run
-            assert next(client.serviceHostIDsInBatches(), None) is not None
+            if next(client.serviceHostIDsInBatches(), None) is None:
+                raise RuntimeError("No more services want to run.")
 
             # Mark the service job updated so we don't stop here.
             self._messages.publish(JobUpdatedMessage(client_id, 1))
@@ -694,8 +707,9 @@ class Leader:
             if exitStatus == 0:
                 logger.debug('Job ended: %s', updatedJob)
             else:
-                logger.warning(f'Job failed with exit value {exitStatus}: {updatedJob}\n'
-                               f'Exit reason: {exitReason}')
+                status_string = str(exitStatus) if exitStatus != EXIT_STATUS_UNAVAILABLE_VALUE else "<UNAVAILABLE>"
+                logger.warning(f'Job failed with exit value {status_string}: {updatedJob}\n'
+                               f'Exit reason: {BatchJobExitReason.to_string(exitReason)}')
                 if exitStatus == CWL_UNSUPPORTED_REQUIREMENT_EXIT_CODE:
                     # This is a CWL job informing us that the workflow is
                     # asking things of us that Toil can't do. When we raise an
@@ -704,7 +718,7 @@ class Leader:
                     logger.warning("This indicates an unsupported CWL requirement!")
                     self.recommended_fail_exit_code = CWL_UNSUPPORTED_REQUIREMENT_EXIT_CODE
             # Tell everyone it stopped running.
-            self._messages.publish(JobCompletedMessage(updatedJob.get_job_kind(), updatedJob.jobStoreID, exitStatus))
+            self._messages.publish(JobCompletedMessage(get_job_kind(updatedJob.get_names()), updatedJob.jobStoreID, exitStatus))
             self.process_finished_job(bsID, exitStatus, wall_time=wallTime, exit_reason=exitReason)
 
     def _processLostJobs(self):
@@ -784,13 +798,16 @@ class Leader:
         logger.debug("Finished the main loop: no jobs left to run.")
 
         # Consistency check the toil state
-        assert self._messages.empty(), f"Pending messages at shutdown: {self._messages}"
-        assert self.toilState.successorCounts == {}, f"Jobs waiting on successors at shutdown: {self.toilState.successorCounts}"
-        assert self.toilState.successor_to_predecessors == {}, f"Successors pending for their predecessors at shutdown: {self.toilState.successor_to_predecessors}"
-        assert self.toilState.service_to_client == {}, f"Services pending for their clients at shutdown: {self.toilState.service_to_client}"
-        assert self.toilState.servicesIssued == {}, f"Services running at shutdown: {self.toilState.servicesIssued}"
-        # assert self.toilState.jobsToBeScheduledWithMultiplePredecessors # These are not properly emptied yet
-        # assert self.toilState.hasFailedSuccessors == set() # These are not properly emptied yet
+        if not self._messages.empty():
+            raise RuntimeError(f"Pending messages at shutdown: {self._messages}")
+        if self.toilState.successorCounts != {}:
+            raise RuntimeError(f"Jobs waiting on successors at shutdown: {self.toilState.successorCounts}")
+        if self.toilState.successor_to_predecessors != {}:
+            raise RuntimeError(f"Successors pending for their predecessors at shutdown: {self.toilState.successor_to_predecessors}")
+        if self.toilState.service_to_client != {}:
+            raise RuntimeError(f"Services pending for their clients at shutdown: {self.toilState.service_to_client}")
+        if self.toilState.servicesIssued != {}:
+            raise RuntimeError(f"Services running at shutdown: {self.toilState.servicesIssued}")
 
     def checkForDeadlocks(self):
         """Check if the system is deadlocked running service jobs."""
@@ -865,8 +882,8 @@ class Leader:
     def issueJob(self, jobNode: JobDescription) -> None:
         """Add a job to the queue of jobs currently trying to run."""
         # Never issue the same job multiple times simultaneously
-        assert jobNode.jobStoreID not in self.toilState.jobs_issued, \
-            f"Attempted to issue {jobNode} multiple times simultaneously!"
+        if jobNode.jobStoreID in self.toilState.jobs_issued:
+            raise RuntimeError(f"Attempted to issue {jobNode} multiple times simultaneously!")
 
         workerCommand = [resolveEntryPoint('_toil_worker'),
                          jobNode.jobName,
@@ -907,7 +924,7 @@ class Leader:
                    "%s and %s",
                    jobNode, str(jobBatchSystemID), jobNode.requirements_string())
         # Tell everyone it is issued and the queue size changed
-        self._messages.publish(JobIssuedMessage(jobNode.get_job_kind(), jobNode.jobStoreID, jobBatchSystemID))
+        self._messages.publish(JobIssuedMessage(get_job_kind(jobNode.get_names()), jobNode.jobStoreID, jobBatchSystemID))
         self._messages.publish(QueueSizeMessage(self.getNumberOfJobsIssued()))
         # Tell the user there's another job to do
         self.progress_overall.total += 1
@@ -926,7 +943,8 @@ class Leader:
         """
         # Grab the service job description
         service = self.toilState.get_job(service_id)
-        assert isinstance(service, ServiceJobDescription)
+        if not isinstance(service, ServiceJobDescription):
+            raise RuntimeError("The grabbed service job description is not the right type.")
 
         if service.preemptible:
             self.preemptibleServiceJobsToBeIssued.append(service_id)
@@ -956,7 +974,8 @@ class Leader:
         elif preemptible:
             return self.preemptibleJobsIssued
         else:
-            assert len(self.issued_jobs_by_batch_system_id) >= self.preemptibleJobsIssued
+            if len(self.issued_jobs_by_batch_system_id) < self.preemptibleJobsIssued:
+                raise RuntimeError("Number of jobs issued cannot be negative.")
             return len(self.issued_jobs_by_batch_system_id) - self.preemptibleJobsIssued
 
     def _getStatusHint(self) -> str:
@@ -990,16 +1009,19 @@ class Leader:
 
         :return: Job description as it was issued.
         """
-        assert jobBatchSystemID in self.issued_jobs_by_batch_system_id
+        if jobBatchSystemID not in self.issued_jobs_by_batch_system_id:
+            raise RuntimeError("Job was already removed or was never issued.")
         issuedDesc = self.toilState.get_job(self.issued_jobs_by_batch_system_id[jobBatchSystemID])
         if issuedDesc.preemptible:
             # len(issued_jobs_by_batch_system_id) should always be greater than or equal to preemptibleJobsIssued,
             # so decrement this value before removing the job from the issuedJob map
-            assert self.preemptibleJobsIssued > 0
+            if self.preemptibleJobsIssued <= 0:
+                raise RuntimeError("The number of preemptive issued jobs cannot be negative.")
             self.preemptibleJobsIssued -= 1
         # It's not issued anymore.
         del self.issued_jobs_by_batch_system_id[jobBatchSystemID]
-        assert issuedDesc.jobStoreID in self.toilState.jobs_issued, f"Job {issuedDesc} came back without being issued"
+        if issuedDesc.jobStoreID not in self.toilState.jobs_issued:
+            raise RuntimeError(f"Job {issuedDesc} came back without being issued")
         self.toilState.jobs_issued.remove(issuedDesc.jobStoreID)
         # If service job
         if issuedDesc.jobStoreID in self.toilState.service_to_client:
@@ -1090,8 +1112,9 @@ class Leader:
         for jobBatchSystemID in missingJobIDsSet.difference(jobBatchSystemIDsSet):
             self.reissueMissingJobs_missingHash.pop(jobBatchSystemID)
             logger.warning("Batch system id: %s is no longer missing", str(jobBatchSystemID))
-        assert issuedJobs.issubset(jobBatchSystemIDsSet) #Assert checks we have
-        #no unexpected jobs running
+        # checks we have no unexpected jobs running
+        if not issuedJobs.issubset(jobBatchSystemIDsSet):
+            raise RuntimeError("An unexpected job is still running.")
         jobsToKill = []
         for jobBatchSystemID in set(jobBatchSystemIDsSet.difference(issuedJobs)):
             jobStoreID = self.issued_jobs_by_batch_system_id[jobBatchSystemID]
@@ -1137,7 +1160,7 @@ class Leader:
             self.progress_overall.update(incr=-1)
             self.progress_failed.update(incr=1)
 
-        # Delegate to the vers
+        # Delegate to the version that uses a JobDescription
         return self.process_finished_job_description(issued_job, result_status, wall_time, exit_reason, batch_system_id)
 
     def process_finished_job_description(self, finished_job: JobDescription, result_status: int,
@@ -1188,11 +1211,12 @@ class Leader:
                     # more memory efficient than read().striplines() while leaving off the
                     # trailing \n left when using readlines()
                     # http://stackoverflow.com/a/15233739
-                    StatsAndLogging.logWithFormatting(job_store_id, log_stream, method=logger.warning,
+                    StatsAndLogging.logWithFormatting(f'Log from job "{job_store_id}"', log_stream, method=logger.warning,
                                                       message='The job seems to have left a log file, indicating failure: %s' % replacement_job)
                 if self.config.writeLogs or self.config.writeLogsGzip:
                     with replacement_job.getLogFileHandle(self.jobStore) as log_stream:
-                        StatsAndLogging.writeLogFiles(replacement_job.chainedJobs, log_stream, self.config, failed=True)
+                        # Send log data from the job store to each per-job log file involved.
+                        StatsAndLogging.writeLogFiles([names.stats_name for names in replacement_job.get_chain()], log_stream, self.config, failed=True)
             if result_status != 0:
                 # If the batch system returned a non-zero exit code then the worker
                 # is assumed not to have captured the failure of the job, so we
@@ -1216,13 +1240,12 @@ class Leader:
                         else:
                             with log_stream:
                                 if os.path.getsize(log_file) > 0:
-                                    StatsAndLogging.logWithFormatting(job_store_id, log_stream, method=logger.warning,
+                                    StatsAndLogging.logWithFormatting(f'Log from job "{job_store_id}"', log_stream, method=logger.warning,
                                                                       message='The batch system left a non-empty file %s:' % log_file)
                                     if self.config.writeLogs or self.config.writeLogsGzip:
                                         file_root, _ = os.path.splitext(os.path.basename(log_file))
-                                        job_names = replacement_job.chainedJobs
-                                        if job_names is None:   # For jobs that fail this way, replacement_job.chainedJobs is not guaranteed to be set
-                                            job_names = [str(replacement_job)]
+                                        job_names = [names.stats_name for names in replacement_job.get_chain()]
+                                        # Tack the batch system log file name onto each job's name
                                         job_names = [j + '_' + file_root for j in job_names]
                                         log_stream.seek(0)
                                         StatsAndLogging.writeLogFiles(job_names, log_stream, self.config, failed=True)
@@ -1289,18 +1312,21 @@ class Leader:
 
         # Tell everyone it failed
 
-        self._messages.publish(JobFailedMessage(job_desc.get_job_kind(), job_id))
+        self._messages.publish(JobFailedMessage(get_job_kind(job_desc.get_names()), job_id))
 
         if job_id in self.toilState.service_to_client:
             # Is a service job
             logger.debug("Service job is being processed as a totally failed job: %s", job_desc)
 
-            assert isinstance(job_desc, ServiceJobDescription)
+
+            if not isinstance(job_desc, ServiceJobDescription):
+                raise RuntimeError("The service job description type is incorrect.")
 
             # Grab the client, which is the predecessor.
             client_id = self.toilState.service_to_client[job_id]
 
-            assert client_id in self.toilState.servicesIssued
+            if client_id not in self.toilState.servicesIssued:
+                raise RuntimeError("The client was never issued.")
 
             # Leave the service job as a service of its predecessor, because it
             # didn't work.
@@ -1331,8 +1357,10 @@ class Leader:
             self.jobStore.delete_file(job_desc.startJobStoreID)
         else:
             # Is a non-service job
-            assert job_id not in self.toilState.servicesIssued
-            assert not isinstance(job_desc, ServiceJobDescription)
+            if job_id in self.toilState.servicesIssued:
+                raise RuntimeError("The non-service job should not have been issued.")
+            if isinstance(job_desc, ServiceJobDescription):
+                raise RuntimeError("The job description type is incorrect.")
 
             # Traverse failed job's successor graph and get the jobStoreID of new successors.
             # Any successor already in toilState.failedSuccessors will not be traversed
@@ -1401,11 +1429,13 @@ class Leader:
                              len(self.toilState.servicesIssued[client_id]))
         elif jobStoreID not in self.toilState.successor_to_predecessors:
             #We have reach the root job
-            assert self._messages.count(JobUpdatedMessage) == 0, "Root job is done but other jobs are still updated"
-            assert len(self.toilState.successor_to_predecessors) == 0, \
-                ("Job {} is finished and had no predecessor, but we have other outstanding jobs "
+            if self._messages.count(JobUpdatedMessage) != 0:
+                raise RuntimeError("Root job is done but other jobs are still updated")
+            if len(self.toilState.successor_to_predecessors) != 0:
+                raise RuntimeError("Job {} is finished and had no predecessor, but we have other outstanding jobs "
                  "with predecessors: {}".format(jobStoreID, self.toilState.successor_to_predecessors.keys()))
-            assert len(self.toilState.successorCounts) == 0, f"Root job is done but jobs waiting on successors: {self.toilState.successorCounts}"
+            if len(self.toilState.successorCounts) != 0:
+                raise RuntimeError("Root job is done but jobs waiting on successors: {self.toilState.successorCounts}")
             logger.debug("Reached root job %s so no predecessors to clean up" % jobStoreID)
 
         else:
@@ -1414,7 +1444,8 @@ class Leader:
 
             # For each predecessor
             for predecessor_id in self.toilState.successor_to_predecessors.pop(jobStoreID):
-                assert isinstance(predecessor_id, str), f"Predecessor ID should be str but is {type(predecessor_id)}"
+                if not isinstance(predecessor_id, str):
+                    raise RuntimeError("Predecessor ID should be str but is {type(predecessor_id)}")
                 predecessor = self.toilState.get_job(predecessor_id)
 
                 # Tell the predecessor that this job is done (keep only other successor jobs)
