@@ -2,7 +2,6 @@ import os
 import re
 import shutil
 import subprocess
-import tempfile
 import sys
 import unittest
 
@@ -70,8 +69,8 @@ class ToilDocumentationTest(ToilTest):
     def testCwlexample(self):
         self.checkExitCode("tutorial_cwlexample.py")
 
-    def testDiscoverfiles(self):
-        self.checkExitCode("tutorial_discoverfiles.py")
+    def testStats(self):
+        self.checkExitCode("tutorial_stats.py")
 
     def testDynamic(self):
         self.checkExitCode("tutorial_dynamic.py")
