@@ -16,12 +16,14 @@ import logging
 import os
 import random
 import sys
-import tempfile
 from uuid import uuid4
 
 from toil.common import getNodeID
 from toil.lib.exceptions import panic, raise_
-from toil.lib.io import AtomicFileCreate, atomic_install, atomic_tmp_file
+from toil.lib.io import (AtomicFileCreate,
+                         atomic_install,
+                         atomic_tmp_file,
+                         mkdtemp)
 from toil.lib.misc import CalledProcessErrorStderr, call_command
 from toil.test import ToilTest, slow
 
@@ -63,7 +65,7 @@ class MiscTests(ToilTest):
         files = {}
         # Create a random directory structure
         for i in range(0,10):
-            directories.append(tempfile.mkdtemp(dir=random.choice(directories), prefix='test'))
+            directories.append(mkdtemp(dir=random.choice(directories), prefix='test'))
         # Create 50 random file entries in different locations in the directories. 75% of the time
         # these are fresh files of size [1, 10] MB and 25% of the time they are hard links to old
         # files.
