@@ -209,7 +209,7 @@ class ResourceTest(ToilTest):
         """
 
         def script():
-            import argparse
+            from configargparse import ArgumentParser
 
             from toil.common import Toil
             from toil.job import Job
@@ -218,7 +218,7 @@ class ResourceTest(ToilTest):
                 pass
 
             if __name__ == '__main__':
-                parser = argparse.ArgumentParser()
+                parser = ArgumentParser()
                 Job.Runner.addToilOptions(parser)
                 options = parser.parse_args()
                 job = Job.wrapFn(fn, memory='10M', cores=0.1, disk='10M')
