@@ -21,7 +21,7 @@ from toil.statsAndLogging import set_logging_from_options
 logger = logging.getLogger(__name__)
 
 def main() -> None:
-    parser = parser_with_common_options(provisioner_options=True, jobstore_option=False)
+    parser = parser_with_common_options(provisioner_options=True, jobstore_option=False, prog="toil destroy-cluster")
     options = parser.parse_args()
     set_logging_from_options(options)
 
